@@ -94,6 +94,7 @@ def resource_allocation_index(G, ebunch=None):
     return _apply_prediction(G, predict, ebunch)
 
 
+@nx._dispatch
 @not_implemented_for("directed")
 @not_implemented_for("multigraph")
 def jaccard_coefficient(G, ebunch=None):
@@ -209,7 +210,7 @@ def adamic_adar_index(G, ebunch=None):
 @not_implemented_for("multigraph")
 def common_neighbor_centrality(G, ebunch=None, alpha=0.8):
     r"""Return the CCPA score for each pair of nodes.
-    
+
     Compute the Common Neighbor and Centrality based Parameterized Algorithm(CCPA)
     score of all node pairs in ebunch.
 
@@ -244,10 +245,11 @@ def common_neighbor_centrality(G, ebunch=None, alpha=0.8):
         2-tuples (u, v) where u and v are nodes in the graph. If ebunch
         is None then all non-existent edges in the graph will be used.
         Default value: None.
-    
-    alpha : Parameter defined for participation of Common Neighbor 
-            and Centrality Algorithm share. Default value set to 0.8
-            because author found better performance at 0.8 for all the 
+
+    alpha : Parameter defined for participation of Common Neighbor
+            and Centrality Algorithm share. Values for alpha should
+            normally be between 0 and 1. Default value set to 0.8
+            because author found better performance at 0.8 for all the
             dataset.
             Default value: 0.8
 
@@ -256,7 +258,7 @@ def common_neighbor_centrality(G, ebunch=None, alpha=0.8):
     -------
     piter : iterator
         An iterator of 3-tuples in the form (u, v, p) where (u, v) is a
-        pair of nodes and p is their Common Neighbor and Centrality based 
+        pair of nodes and p is their Common Neighbor and Centrality based
         Parameterized Algorithm(CCPA) score.
 
     Examples
@@ -270,17 +272,33 @@ def common_neighbor_centrality(G, ebunch=None, alpha=0.8):
 
     References
     ----------
-    .. [1] Ahmad, I., Akhtar, M.U., Noor, S. et al. 
-           Missing Link Prediction using Common Neighbor and Centrality based Parameterized Algorithm. 
-           Sci Rep 10, 364 (2020). 
+    .. [1] Ahmad, I., Akhtar, M.U., Noor, S. et al.
+           Missing Link Prediction using Common Neighbor and Centrality based Parameterized Algorithm.
+           Sci Rep 10, 364 (2020).
            https://doi.org/10.1038/s41598-019-57304-y
     """
-    shortest_path = nx.shortest_path(G)
 
-    def predict(u, v):
-        return alpha * len(list(nx.common_neighbors(G, u, v))) + (1 - alpha) * (
-            G.number_of_nodes() / (len(shortest_path[u][v]) - 1)
-        )
+    # When alpha == 1, the CCPA score simplifies to the number of common neighbors.
+    if alpha == 1:
+
+        def predict(u, v):
+            if u == v:
+                raise nx.NetworkXAlgorithmError("Self links are not supported")
+
+            return sum(1 for _ in nx.common_neighbors(G, u, v))
+
+    else:
+        spl = dict(nx.shortest_path_length(G))
+        inf = float("inf")
+
+        def predict(u, v):
+            if u == v:
+                raise nx.NetworkXAlgorithmError("Self links are not supported")
+            path_len = spl[u].get(v, inf)
+
+            return alpha * sum(1 for _ in nx.common_neighbors(G, u, v)) + (
+                1 - alpha
+            ) * (G.number_of_nodes() / path_len)
 
     return _apply_prediction(G, predict, ebunch)
 
@@ -575,5 +593,5 @@ def _community(G, u, community):
     node_u = G.nodes[u]
     try:
         return node_u[community]
-    except KeyError as e:
-        raise nx.NetworkXAlgorithmError("No community information") from e
+    except KeyError as err:
+        raise nx.NetworkXAlgorithmError("No community information") from err
