@@ -14,98 +14,65 @@ __all__ = [
 ]
 
 
-def union(G, H, rename=(None, None), name=None):
-    """ Return the union of graphs G and H.
+def union(G, H, rename=()):
+    """Combine graphs G and H. The names of nodes must be unique.
+
+    A name collision between the graphs will raise an exception.
+
+    A renaming facility is provided to avoid name collisions.
 
-    Graphs G and H must be disjoint, otherwise an exception is raised.
 
     Parameters
     ----------
-    G,H : graph
+    G, H : graph
        A NetworkX graph
 
-    rename : bool , default=(None, None)
+    rename : iterable , optional
        Node names of G and H can be changed by specifying the tuple
        rename=('G-','H-') (for example).  Node "u" in G is then renamed
        "G-u" and "v" in H is renamed "H-v".
 
-    name : string
-       Specify the name for the union graph
-
     Returns
     -------
     U : A union graph with the same type as G.
 
-    Notes
-    -----
-    To force a disjoint union with node relabeling, use
-    disjoint_union(G,H) or convert_node_labels_to integers().
-
-    Graph, edge, and node attributes are propagated from G and H
-    to the union graph.  If a graph attribute is present in both
-    G and H the value from H is used.
-
     See Also
     --------
+    compose
+    :func:`~networkx.Graph.update`
     disjoint_union
+
+    Notes
+    -----
+    To combine graphs that have common nodes, consider compose(G, H)
+    or the method, Graph.update().
+
+    disjoint_union() is similar to union() except that it avoids name clashes
+    by relabeling the nodes with sequential integers.
+
+    Edge and node attributes are propagated from G and H to the union graph.
+    Graph attributes are also propagated, but if they are present in both G and H,
+    then the value from H is used.
+
+    Examples
+    --------
+    >>> G = nx.Graph([(0, 1), (0, 2), (1, 2)])
+    >>> H = nx.Graph([(0, 1), (0, 3), (1, 3), (1, 2)])
+    >>> U = nx.union(G, H, rename=("G", "H"))
+    >>> U.nodes
+    NodeView(('G0', 'G1', 'G2', 'H0', 'H1', 'H3', 'H2'))
+    >>> U.edges
+    EdgeView([('G0', 'G1'), ('G0', 'G2'), ('G1', 'G2'), ('H0', 'H1'), ('H0', 'H3'), ('H1', 'H3'), ('H1', 'H2')])
+
+
     """
-    if not G.is_multigraph() == H.is_multigraph():
-        raise nx.NetworkXError("G and H must both be graphs or multigraphs.")
-    # Union is the same type as G
-    R = G.__class__()
-    # add graph attributes, H attributes take precedent over G attributes
-    R.graph.update(G.graph)
-    R.graph.update(H.graph)
-
-    # rename graph to obtain disjoint node labels
-    def add_prefix(graph, prefix):
-        if prefix is None:
-            return graph
-
-        def label(x):
-            if isinstance(x, str):
-                name = prefix + x
-            else:
-                name = prefix + repr(x)
-            return name
-
-        return nx.relabel_nodes(graph, label)
-
-    G = add_prefix(G, rename[0])
-    H = add_prefix(H, rename[1])
-    if set(G) & set(H):
-        raise nx.NetworkXError(
-            "The node sets of G and H are not disjoint.",
-            "Use appropriate rename=(Gprefix,Hprefix)" "or use disjoint_union(G,H).",
-        )
-    if G.is_multigraph():
-        G_edges = G.edges(keys=True, data=True)
-    else:
-        G_edges = G.edges(data=True)
-    if H.is_multigraph():
-        H_edges = H.edges(keys=True, data=True)
-    else:
-        H_edges = H.edges(data=True)
-
-    # add nodes
-    R.add_nodes_from(G)
-    R.add_nodes_from(H)
-    # add edges
-    R.add_edges_from(G_edges)
-    R.add_edges_from(H_edges)
-    # add node attributes
-    for n in G:
-        R.nodes[n].update(G.nodes[n])
-    for n in H:
-        R.nodes[n].update(H.nodes[n])
-
-    return R
+    return nx.union_all([G, H], rename)
 
 
 def disjoint_union(G, H):
-    """ Return the disjoint union of graphs G and H.
+    """Combine graphs G and H. The nodes are assumed to be unique (disjoint).
 
-    This algorithm forces distinct integer node labels.
+    This algorithm automatically relabels nodes to avoid name collisions.
 
     Parameters
     ----------
@@ -116,6 +83,12 @@ def disjoint_union(G, H):
     -------
     U : A union graph with the same type as G.
 
+    See Also
+    --------
+    union
+    compose
+    :func:`~networkx.Graph.update`
+
     Notes
     -----
     A new graph is created, of the same class as G.  It is recommended
@@ -124,28 +97,44 @@ def disjoint_union(G, H):
     The nodes of G are relabeled 0 to len(G)-1, and the nodes of H are
     relabeled len(G) to len(G)+len(H)-1.
 
-    Graph, edge, and node attributes are propagated from G and H
-    to the union graph.  If a graph attribute is present in both
-    G and H the value from H is used.
+    Renumbering forces G and H to be disjoint, so no exception is ever raised for a name collision.
+    To preserve the check for common nodes, use union().
+
+    Edge and node attributes are propagated from G and H to the union graph.
+    Graph attributes are also propagated, but if they are present in both G and H,
+    then the value from H is used.
+
+    To combine graphs that have common nodes, consider compose(G, H)
+    or the method, Graph.update().
+
+    Examples
+    --------
+    >>> G = nx.Graph([(0, 1), (0, 2), (1, 2)])
+    >>> H = nx.Graph([(0, 3), (1, 2), (2, 3)])
+    >>> G.nodes[0]["key1"] = 5
+    >>> H.nodes[0]["key2"] = 10
+    >>> U = nx.disjoint_union(G, H)
+    >>> U.nodes(data=True)
+    NodeDataView({0: {'key1': 5}, 1: {}, 2: {}, 3: {'key2': 10}, 4: {}, 5: {}, 6: {}})
+    >>> U.edges
+    EdgeView([(0, 1), (0, 2), (1, 2), (3, 4), (4, 6), (5, 6)])
     """
-    R1 = nx.convert_node_labels_to_integers(G)
-    R2 = nx.convert_node_labels_to_integers(H, first_label=len(R1))
-    R = union(R1, R2)
-    R.graph.update(G.graph)
-    R.graph.update(H.graph)
-    return R
+    return nx.disjoint_union_all([G, H])
 
 
 def intersection(G, H):
-    """Returns a new graph that contains only the edges that exist in
+    """Returns a new graph that contains only the nodes and the edges that exist in
     both G and H.
 
-    The node sets of H and G must be the same.
-
     Parameters
     ----------
     G,H : graph
-       A NetworkX graph.  G and H must have the same node sets.
+       A NetworkX graph. G and H can have different node sets but must be both graphs or both multigraphs.
+
+    Raises
+    ------
+    NetworkXError
+        If one is a MultiGraph and the other one is a graph.
 
     Returns
     -------
@@ -162,32 +151,19 @@ def intersection(G, H):
     >>> H = nx.path_graph(5)
     >>> R = G.copy()
     >>> R.remove_nodes_from(n for n in G if n not in H)
+    >>> R.remove_edges_from(e for e in G.edges if e not in H.edges)
+
+    Examples
+    --------
+    >>> G = nx.Graph([(0, 1), (0, 2), (1, 2)])
+    >>> H = nx.Graph([(0, 3), (1, 2), (2, 3)])
+    >>> R = nx.intersection(G, H)
+    >>> R.nodes
+    NodeView((0, 1, 2))
+    >>> R.edges
+    EdgeView([(1, 2)])
     """
-    # create new graph
-    R = nx.create_empty_copy(G)
-
-    if not G.is_multigraph() == H.is_multigraph():
-        raise nx.NetworkXError("G and H must both be graphs or multigraphs.")
-    if set(G) != set(H):
-        raise nx.NetworkXError("Node sets of graphs are not equal")
-
-    if G.number_of_edges() <= H.number_of_edges():
-        if G.is_multigraph():
-            edges = G.edges(keys=True)
-        else:
-            edges = G.edges()
-        for e in edges:
-            if H.has_edge(*e):
-                R.add_edge(*e)
-    else:
-        if H.is_multigraph():
-            edges = H.edges(keys=True)
-        else:
-            edges = H.edges()
-        for e in edges:
-            if G.has_edge(*e):
-                R.add_edge(*e)
-    return R
+    return nx.intersection_all([G, H])
 
 
 def difference(G, H):
@@ -198,7 +174,7 @@ def difference(G, H):
     Parameters
     ----------
     G,H : graph
-       A NetworkX graph.  G and H must have the same node sets.
+       A NetworkX graph. G and H must have the same node sets.
 
     Returns
     -------
@@ -208,13 +184,23 @@ def difference(G, H):
     -----
     Attributes from the graph, nodes, and edges are not copied to the new
     graph.  If you want a new graph of the difference of G and H with
-    with the attributes (including edge data) from G use remove_nodes_from()
+    the attributes (including edge data) from G use remove_nodes_from()
     as follows:
 
     >>> G = nx.path_graph(3)
     >>> H = nx.path_graph(5)
     >>> R = G.copy()
     >>> R.remove_nodes_from(n for n in G if n in H)
+
+    Examples
+    --------
+    >>> G = nx.Graph([(0, 1), (0, 2), (1, 2), (1, 3)])
+    >>> H = nx.Graph([(0, 1), (1, 2), (0, 3)])
+    >>> R = nx.difference(G, H)
+    >>> R.nodes
+    NodeView((0, 1, 2, 3))
+    >>> R.edges
+    EdgeView([(0, 2), (1, 3)])
     """
     # create new graph
     if not G.is_multigraph() == H.is_multigraph():
@@ -252,6 +238,16 @@ def symmetric_difference(G, H):
     -----
     Attributes from the graph, nodes, and edges are not copied to the new
     graph.
+
+    Examples
+    --------
+    >>> G = nx.Graph([(0, 1), (0, 2), (1, 2), (1, 3)])
+    >>> H = nx.Graph([(0, 1), (1, 2), (0, 3)])
+    >>> R = nx.symmetric_difference(G, H)
+    >>> R.nodes
+    NodeView((0, 1, 2, 3))
+    >>> R.edges
+    EdgeView([(0, 2), (0, 3), (1, 3)])
     """
     # create new graph
     if not G.is_multigraph() == H.is_multigraph():
@@ -287,10 +283,12 @@ def symmetric_difference(G, H):
 
 
 def compose(G, H):
-    """Returns a new graph of G composed with H.
+    """Compose graph G with H by combining nodes and edges into a single graph.
 
-    Composition is the simple union of the node sets and edge sets.
-    The node sets of G and H do not need to be disjoint.
+    The node sets and edges sets do not need to be disjoint.
+
+    Composing preserves the attributes of nodes and edges.
+    Attribute values from H take precedent over attribute values from G.
 
     Parameters
     ----------
@@ -299,37 +297,65 @@ def compose(G, H):
 
     Returns
     -------
-    C: A new graph  with the same type as G
+    C: A new graph with the same type as G
+
+    See Also
+    --------
+    :func:`~networkx.Graph.update`
+    union
+    disjoint_union
 
     Notes
     -----
     It is recommended that G and H be either both directed or both undirected.
-    Attributes from H take precedent over attributes from G.
 
     For MultiGraphs, the edges are identified by incident nodes AND edge-key.
     This can cause surprises (i.e., edge `(1, 2)` may or may not be the same
     in two graphs) if you use MultiGraph without keeping track of edge keys.
+
+    If combining the attributes of common nodes is not desired, consider union(),
+    which raises an exception for name collisions.
+
+    Examples
+    --------
+    >>> G = nx.Graph([(0, 1), (0, 2)])
+    >>> H = nx.Graph([(0, 1), (1, 2)])
+    >>> R = nx.compose(G, H)
+    >>> R.nodes
+    NodeView((0, 1, 2))
+    >>> R.edges
+    EdgeView([(0, 1), (0, 2), (1, 2)])
+
+    By default, the attributes from `H` take precedent over attributes from `G`.
+    If you prefer another way of combining attributes, you can update them after the compose operation:
+
+    >>> G = nx.Graph([(0, 1, {'weight': 2.0}), (3, 0, {'weight': 100.0})])
+    >>> H = nx.Graph([(0, 1, {'weight': 10.0}), (1, 2, {'weight': -1.0})])
+    >>> nx.set_node_attributes(G, {0: 'dark', 1: 'light', 3: 'black'}, name='color')
+    >>> nx.set_node_attributes(H, {0: 'green', 1: 'orange', 2: 'yellow'}, name='color')
+    >>> GcomposeH = nx.compose(G, H)
+
+    Normally, color attribute values of nodes of GcomposeH come from H. We can workaround this as follows:
+
+    >>> node_data = {n: G.nodes[n]['color'] + " " + H.nodes[n]['color'] for n in G.nodes & H.nodes}
+    >>> nx.set_node_attributes(GcomposeH, node_data, 'color')
+    >>> print(GcomposeH.nodes[0]['color'])
+    dark green
+
+    >>> print(GcomposeH.nodes[3]['color'])
+    black
+
+    Similarly, we can update edge attributes after the compose operation in a way we prefer:
+
+    >>> edge_data = {e: G.edges[e]['weight'] * H.edges[e]['weight'] for e in G.edges & H.edges}
+    >>> nx.set_edge_attributes(GcomposeH, edge_data, 'weight')
+    >>> print(GcomposeH.edges[(0, 1)]['weight'])
+    20.0
+
+    >>> print(GcomposeH.edges[(3, 0)]['weight'])
+    100.0
     """
-    if not G.is_multigraph() == H.is_multigraph():
-        raise nx.NetworkXError("G and H must both be graphs or multigraphs.")
-
-    R = G.__class__()
-    # add graph attributes, H attributes take precedent over G attributes
-    R.graph.update(G.graph)
-    R.graph.update(H.graph)
-
-    R.add_nodes_from(G.nodes(data=True))
-    R.add_nodes_from(H.nodes(data=True))
-
-    if G.is_multigraph():
-        R.add_edges_from(G.edges(keys=True, data=True))
-    else:
-        R.add_edges_from(G.edges(data=True))
-    if H.is_multigraph():
-        R.add_edges_from(H.edges(keys=True, data=True))
-    else:
-        R.add_edges_from(H.edges(data=True))
-    return R
+    return nx.compose_all([G, H])
 
 
 def full_join(G, H, rename=(None, None)):
@@ -345,7 +371,7 @@ def full_join(G, H, rename=(None, None)):
     G, H : graph
        A NetworkX graph
 
-    rename : bool , default=(None, None)
+    rename : tuple , default=(None, None)
        Node names of G and H can be changed by specifying the tuple
        rename=('G-','H-') (for example).  Node "u" in G is then renamed
        "G-u" and "v" in H is renamed "H-v".
@@ -370,6 +396,16 @@ def full_join(G, H, rename=(None, None)):
     to the union graph.  If a graph attribute is present in both
     G and H the value from H is used.
 
+    Examples
+    --------
+    >>> G = nx.Graph([(0, 1), (0, 2)])
+    >>> H = nx.Graph([(3, 4)])
+    >>> R = nx.full_join(G, H, rename=("G", "H"))
+    >>> R.nodes
+    NodeView(('G0', 'G1', 'G2', 'H3', 'H4'))
+    >>> R.edges
+    EdgeView([('G0', 'G1'), ('G0', 'G2'), ('G0', 'H3'), ('G0', 'H4'), ('G1', 'H3'), ('G1', 'H4'), ('G2', 'H3'), ('G2', 'H4'), ('H3', 'H4')])
+
     See Also
     --------
     union
@@ -382,11 +418,7 @@ def full_join(G, H, rename=(None, None)):
             return graph
 
         def label(x):
-            if isinstance(x, str):
-                name = prefix + x
-            else:
-                name = prefix + repr(x)
-            return name
+            return f"{prefix}{x}"
 
         return nx.relabel_nodes(graph, label)
 
