@@ -32,8 +32,8 @@ __all__ = [
     "read_multiline_adjlist",
 ]
 
-from networkx.utils import open_file
 import networkx as nx
+from networkx.utils import open_file
 
 
 def generate_multiline_adjlist(G, delimiter=" "):
@@ -135,12 +135,16 @@ def generate_multiline_adjlist(G, delimiter=" "):
 
 @open_file(1, mode="wb")
 def write_multiline_adjlist(G, path, delimiter=" ", comments="#", encoding="utf-8"):
-    """ Write the graph G in multiline adjacency list format to path
+    """Write the graph G in multiline adjacency list format to path
 
     Parameters
     ----------
     G : NetworkX graph
 
+    path : string or file
+       Filename or file handle to write to.
+       Filenames ending in .gz or .bz2 will be compressed.
+
     comments : string, optional
        Marker for comment lines
 
@@ -203,6 +207,9 @@ def parse_multiline_adjlist(
     nodetype : Python type, optional
        Convert nodes to this type.
 
+    edgetype : Python type, optional
+       Convert edges to this type.
+
     comments : string, optional
        Marker for comment lines
 
@@ -240,23 +247,23 @@ def parse_multiline_adjlist(
         try:
             (u, deg) = line.strip().split(delimiter)
             deg = int(deg)
-        except BaseException as e:
-            raise TypeError(f"Failed to read node and degree on line ({line})") from e
+        except BaseException as err:
+            raise TypeError(f"Failed to read node and degree on line ({line})") from err
         if nodetype is not None:
             try:
                 u = nodetype(u)
-            except BaseException as e:
+            except BaseException as err:
                 raise TypeError(
                     f"Failed to convert node ({u}) to " f"type {nodetype}"
-                ) from e
+                ) from err
         G.add_node(u)
         for i in range(deg):
             while True:
                 try:
                     line = next(lines)
-                except StopIteration as e:
+                except StopIteration as err:
                     msg = f"Failed to find neighbor for node ({u})"
-                    raise TypeError(msg) from e
+                    raise TypeError(msg) from err
                 p = line.find(comments)
                 if p >= 0:
                     line = line[:p]
@@ -271,17 +278,17 @@ def parse_multiline_adjlist(
             if nodetype is not None:
                 try:
                     v = nodetype(v)
-                except BaseException as e:
+                except BaseException as err:
                     raise TypeError(
                         f"Failed to convert node ({v}) " f"to type {nodetype}"
-                    ) from e
+                    ) from err
             if edgetype is not None:
                 try:
                     edgedata = {"weight": edgetype(data)}
-                except BaseException as e:
+                except BaseException as err:
                     raise TypeError(
                         f"Failed to convert edge data ({data}) " f"to type {edgetype}"
-                    ) from e
+                    ) from err
             else:
                 try:  # try to evaluate
                     edgedata = literal_eval(data)
